from datetime import datetime
from typing import Annotated

from pydantic import Field

from ..base import VulnersApiProxy, endpoint


class ArchiveApi(VulnersApiProxy):
    fetch_collection = endpoint(
        "ArchiveApi.fetch_collection",
        method="GET",
        url="/api/v4/archive/collection",
        description="Get entire collection data",
        params={"type": str},
    )

    fetch_collection_update = endpoint(
        "ArchiveApi.fetch_collection_update",
        method="GET",
        url="/api/v4/archive/collection-update",
        description="Get collection updates only",
        params={"type": str, "after": datetime},
    )

    get_collection = endpoint(
        "ArchiveApi.get_collection",
        method="GET",
        url="/api/v3/archive/collection/",
        params={
            "type": str,
            "datefrom": Annotated[str, Field(default="1976-01-01")],
            "dateto": Annotated[str, Field(default="2199-01-01")],
        },
        deprecated=(
            "ArchiveApi.get_collection() is deprecated and will be removed in future releases.\n"
        ),
    )

    get_distributive = endpoint(
        "ArchiveApi.get_distributive",
        method="GET",
        url="/api/v3/archive/distributive/",
        params={"os": str, "version": str},
        deprecated=(
            "ArchiveApi.get_distributive() is deprecated and will be removed in future releases."
        ),
        response_handler=lambda c: [o["_source"] for o in c],
        timeout=120,
    )

    getsploit = endpoint(
        "ArchiveApi.getsploit",
        method="GET",
        url="/api/v3/archive/getsploit/",
        deprecated=(
            "ArchiveApi.getsploit() is deprecated and will be removed in future releases."
        ),
        parse_response=False,
    )



from typing import Annotated, Literal

from pydantic import Field

from ..base import Unset, VulnersApiProxy, endpoint


class MiscApi(VulnersApiProxy):
    get_suggestion = endpoint(
        "MiscApi.get_suggestion",
        method="POST",
        url="/api/v3/search/suggest/",
        params={
            "field_name": Annotated[str, Field(alias="fieldName")],
            "type": Annotated[Literal["distinct"], Field(default="distinct")],
        },
        response_handler=lambda c: c["suggest"],
    )

    get_ai_score = endpoint(
        "MiscApi.get_ai_score",
        method="POST",
        url="/api/v3/ai/scoretext/",
        params={"text": str},
        response_handler=lambda c: c.get("score", 0),
    )

    search_cpe = endpoint(
        "MiscApi.search_cpe",
        method="GET",
        url="/api/v4/search/cpe",
        description="Search CPE",
        params={
            "product": Annotated[str, Field(description="Product string to search CPE for")],
            "vendor": Annotated[
                str, Field(default=Unset, description="Optional vendor to include in CPE")
            ],
        },
        response_handler=lambda c: c["result"],
    )

    query_autocomplete = endpoint(
        "MiscApi.query_autocomplete",
        method="POST",
        url="/api/v3/search/autocomplete/",
        description="Ask Vulners for possible suggestions to complete your query",
        params={"query": str},
        response_handler=lambda c: [q[0] for q in c["suggestions"]],
    )

    get_web_application_rules = endpoint(
        "MiscApi.get_web_application_rules",
        method="GET",
        url="/api/v3/burp/rules/",
        deprecated=(
            "MiscApi.get_web_application_rules() is deprecated "
            "and will be removed in future releases."
        ),
    )



from __future__ import annotations

import base64
import uuid
from typing import Annotated, Any, Iterable, Iterator, Literal, Mapping

from pydantic import Field
from typing_extensions import TypedDict

from .base import Unset, VulnersApiBase, endpoint


class NotificationObj(TypedDict):
    period: Literal["disabled", "asap", "hourly", "daily"]
    email: list[str]
    slack: list[str]


class ApiObject(Mapping[str, Any]):
    __slots__ = ("_api", "__dict__")

    def __init__(self, api: VScannerApi, data: dict[str, Any]):
        self._api = api
        self.__dict__ = data

    def __len__(self):
        return len(self.__dict__)

    def __getitem__(self, k):
        return self.__dict__[k]

    def __iter__(self) -> Iterator[Any]:
        return iter(self.__dict__)

    def __repr__(self) -> str:
        return repr(self.__dict__)


class ProjectList(list):
    def __init__(self, api: VScannerApi, value: Iterable[dict[str, Any]]) -> None:
        super().__init__(Project(api, c) for c in value)


class Project(ApiObject):
    _id: uuid.UUID
    name: str
    license_id: str
    notification: NotificationObj
    result_expire_in: int | None

    def update(self, **kwargs):
        kwargs.setdefault("name", self.name)
        kwargs.setdefault("license_id", self.license_id)
        kwargs.setdefault("notification", self.notification)
        kwargs.setdefault("result_expire_in", self.result_expire_in)
        self.__dict__ = self._api.update_project(self._id, **kwargs).__dict__

    def delete(self):
        self._api.delete_project(self._id)

    def get_tasks(self, *args, **kwargs):
        return self._api.get_tasks(self._id, *args, **kwargs)

    def create_task(self, *args, **kwargs):
        return self._api.create_task(self._id, *args, **kwargs)

    def get_results(self, *args, **kwargs):
        return self._api.get_results(self._id, *args, **kwargs)

    def get_statistics(self, *args, **kwargs):
        return self._api.get_statistics(self._id, *args, **kwargs)


class TaskList(list):
    def __init__(self, api: VScannerApi, value: Iterable[dict[str, Any]]) -> None:
        super().__init__(Task(api, c) for c in value)


class Task(ApiObject):
    _id: uuid.UUID
    name: str
    project_id: uuid.UUID
    networks: list[str]
    ports: list[str]
    timing: str
    schedule: str
    enabled: bool
    context_id: uuid.UUID

    def update(self, **kwargs):
        kwargs.setdefault("name", self.name)
        kwargs.setdefault("networks", self.networks)
        kwargs.setdefault("ports", self.ports)
        kwargs.setdefault("timing", self.timing)
        kwargs.setdefault("schedule", self.schedule)
        kwargs.setdefault("enabled", self.enabled)
        self.__dict__ = self._api.update_task(self.project_id, self._id, **kwargs).__dict__

    def delete(self):
        self._api.delete_task(self.project_id, self._id)

    def start_task(self):
        self.__dict__ = self._api.start_task(self.project_id, self._id).__dict__


class ResultList(list):
    def __init__(self, api: VScannerApi, value: Iterable[dict[str, Any]]) -> None:
        super().__init__(Result(api, c) for c in value)


class Result(ApiObject):
    _id: uuid.UUID
    project_id: uuid.UUID
    screens: Mapping[str, Any]

    def delete(self):
        self._api.delete_result(self.project_id, self._id)

    def get_screenshot(self, port, as_base64=False):
        try:
            screen = self.screens[str(port)]["screen"]
        except (AttributeError, KeyError):
            return None
        if not screen:
            return None
        return self._api.get_image_binary(screen, as_base64)


class VScannerApi(VulnersApiBase):
    _ratelimit_key = "vscanner"

    get_licenses = endpoint(
        "VScannerApi.get_licenses",
        method="GET",
        url="/api/v3/useraction/licenseids",
        description="Get user's license ids.",
        wrapper=ApiObject,
    )
    get_projects = endpoint(
        "VScannerApi.get_projects",
        method="GET",
        url="/api/v3/proxy/vscanner/v2/projects/",
        description="Get existing projects.",
        params={
            "offset": Annotated[int, Field(default=0, ge=0)],
            "limit": Annotated[int, Field(default=50, le=1000)],
        },
        wrapper=ProjectList,
    )
    create_project = endpoint(
        "VScannerApi.create_project",
        method="POST",
        url="/api/v3/proxy/vscanner/v2/projects/",
        description="Create new project.",
        params={
            "name": Annotated[str, Field(description="New project name")],
            "license_id": Annotated[uuid.UUID, Field(description="User's license id")],
            "notification": Annotated[
                NotificationObj,
                Field(
                    description=(
                        "Use VScannerApi.Notification or VScannerApi.DisabledNotification helpers "
                        "to create notification object."
                    ),
                ),
            ],
            "result_expire_in": Annotated[
                int | None,
                Field(
                    default=Unset,
                    gt=0,
                    description="Result expire in N days. Null means it will never expire",
                ),
            ],
        },
        wrapper=Project,
    )
    update_project = endpoint(
        "VScannerApi.update_project",
        method="PUT",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}",
        description="Update existing project.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "name": Annotated[str, Field(description="Project name")],
            "license_id": Annotated[uuid.UUID, Field(description="User's license id")],
            "notification": Annotated[
                NotificationObj,
                Field(
                    description=(
                        "Use VScannerApi.Notification or VScannerApi.DisabledNotification helpers "
                        "to create notification object."
                    ),
                ),
            ],
            "result_expire_in": Annotated[
                int | None,
                Field(
                    gt=0,
                    description="Result expire in N days. Null means it will never expire",
                ),
            ],
        },
        wrapper=Project,
    )
    delete_project = endpoint(
        "VScannerApi.delete_project",
        method="DELETE",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}",
        description="Delete existing project.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
        },
    )
    get_tasks = endpoint(
        "VScannerApi.get_tasks",
        method="GET",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/tasks",
        description="Get project tasks",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "offset": Annotated[int, Field(default=0, ge=0)],
            "limit": Annotated[int, Field(default=50, le=1000)],
        },
        wrapper=TaskList,
    )
    create_task = endpoint(
        "VScannerApi.create_task",
        method="POST",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/tasks",
        description="Create new task.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "name": Annotated[str, Field(description="Task name")],
            "networks": Annotated[
                list[str], Field(description="List of networks (ip or domains)")
            ],
            "ports": Annotated[list[str], Field(description="List of ports or port ranges")],
            "schedule": Annotated[str, Field(description="Crontab string")],
            "timing": Annotated[str, Field(description="Scan timing")],
            "enabled": Annotated[bool, Field(description="Enable/disable task")],
        },
        wrapper=Task,
    )
    update_task = endpoint(
        "VScannerApi.update_task",
        method="PUT",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/tasks/{task_id}",
        description="Update task.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "task_id": Annotated[uuid.UUID, Field(description="Task ID")],
            "name": Annotated[str, Field(description="Task name")],
            "networks": Annotated[
                list[str], Field(description="List of networks (ip or domains)")
            ],
            "ports": Annotated[list[str], Field(description="List of ports or port ranges")],
            "schedule": Annotated[str, Field(description="Crontab string")],
            "timing": Annotated[str, Field(description="Scan timing")],
            "enabled": Annotated[bool, Field(description="Enable/disable task")],
        },
        wrapper=Task,
    )
    start_task = endpoint(
        "VScannerApi.start_task",
        method="POST",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/tasks/{task_id}/start",
        description="Start task asap.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "task_id": Annotated[uuid.UUID, Field(description="Task ID")],
        },
        wrapper=Task,
    )
    delete_task = endpoint(
        "VScannerApi.delete_task",
        method="DELETE",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/tasks/{task_id}",
        description="Delete task.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "task_id": Annotated[uuid.UUID, Field(description="Task ID")],
        },
    )
    get_results = endpoint(
        "VScannerApi.get_results",
        method="GET",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/results",
        description="Get results.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "search": Annotated[
                str, Field(default=Unset, description="Search by ip, network, name or vuln_id.")
            ],
            "in_port": Annotated[list[str], Field(default=Unset, description="Include ports")],
            "ex_port": Annotated[list[str], Field(default=Unset, description="Exclude ports")],
            "min_cvss": Annotated[float, Field(default=Unset, description="Minimum CVSS value")],
            "max_cvss": Annotated[float, Field(default=Unset, description="Maximum CVSS value")],
            "last_seen": Annotated[
                int, Field(default=Unset, description="last_seen >= given value")
            ],
            "first_seen": Annotated[
                int, Field(default=Unset, description="first_seen >= given value")
            ],
            "last_seen_port": Annotated[
                int, Field(default=Unset, description="last_seen_port >= given value")
            ],
            "first_seen_port": Annotated[
                int, Field(default=Unset, description="first_seen_port >= given value")
            ],
            "sort": Annotated[
                Literal[
                    "ip",
                    "name",
                    "last_seen",
                    "first_seen",
                    "resolved",
                    "min_cvss",
                    "max_cvss",
                ],
                Field(default="last_seen", description="Sort by field"),
            ],
            "sort_dir": Annotated[
                Literal["asc", "desc"], Field(default="asc", description="Sort direction")
            ],
            "offset": Annotated[int, Field(default=0, ge=0)],
            "limit": Annotated[int, Field(default=50, le=1000)],
        },
        wrapper=ResultList,
    )
    delete_result = endpoint(
        "VScannerApi.delete_result",
        method="DELETE",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/results/{result_id}",
        description="Delete result by id.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "result_id": Annotated[uuid.UUID, Field(description="Result ID")],
        },
    )
    get_statistics = endpoint(
        "VScannerApi.get_statistics",
        method="GET",
        url="/api/v3/proxy/vscanner/v2/projects/{project_id}/statistic",
        description="Get project statistics.",
        params={
            "project_id": Annotated[uuid.UUID, Field(description="Project ID")],
            "stat": Annotated[
                list[
                    Literal[
                        "total_hosts",
                        "vulnerable_hosts",
                        "unique_cve",
                        "min_max_cvss",
                        "vulnerabilities_rank",
                        "vulnerable_hosts_rank",
                    ]
                ],
                Field(min_length=1, description="List of required aggregations"),
            ],
        },
    )

    def get_image_binary(self, image_uri: str, as_base64: bool = False):
        content = self._invoke(
            "GET", "/vscanner/screen/" + image_uri, {}, (), parse_content=False
        )
        if as_base64:
            return base64.b64encode(content)
        return content

    @staticmethod
    def Notification(
        period: Literal["disabled", "asap", "hourly", "daily"],
        emails: list[str] | None = None,
        slack_webhooks: list[str] | None = None,
    ) -> NotificationObj:
        """
        Create notification project

        period: one of "disabled", "asap", "hourly" or "daily"
        emails: list of emails
        slack_webhooks: list of slack webhooks
        """
        if period not in ("disabled", "asap", "hourly", "daily"):
            raise ValueError(
                'period expected to be one of "disabled", "asap", "hourly" or "daily"'
            )
        return {
            "period": period,
            "email": emails or [],
            "slack": slack_webhooks or [],
        }

    @staticmethod
    def DisabledNotification():
        """
        Create stub notification object with "disabled" period and empty methods
        """
        return {
            "period": "disabled",
            "email": [],
            "slack": [],
        }



from typing import Annotated, Literal, Sequence

from pydantic import Field, FilePath
from typing_extensions import Required, TypedDict

from ..base import Unset, VulnersApiProxy, endpoint


class AuditItem(TypedDict, total=False):
    part: Literal["a", "o", "h"]
    vendor: str
    product: Required[str]
    version: str
    update: str
    edition: str
    language: str
    sw_edition: str
    target_sw: str
    target_hw: str
    other: str


class WinAuditItem(TypedDict):
    software: str
    version: str


AuditFields = Literal[
    "title",
    "short_description",
    "type",
    "published",
    "modified",
    "href",
    "metrics",
    "exploitation",
    "cvelist",
    "ai_score",
    "epss",
    "description",
    "enchantments",
    "webApplicability",
    "cvelistMetrics",
]


class AuditApi(VulnersApiProxy):
    software = endpoint(
        "AuditApi.software",
        method="POST",
        url="/api/v4/audit/software/",
        params={
            "software": Annotated[
                list[AuditItem | str],
                Field(
                    min_length=1,
                    description="List of dicts. E.g., [{'product': 'curl', 'version': '8.11.1', ...}, ...]",
                ),
            ],
            "match": Annotated[Literal["partial", "full"], Field(default="partial")],
            "fields": Annotated[
                Sequence[AuditFields],
                Field(
                    default=Unset,
                    description="List of fields to retrieve about each vulnerability",
                ),
            ],
            "config": Annotated[Sequence[str], Field(default=Unset)],
            "catalog": Annotated[Literal["official", "extended"], Field(default="official")],
        },
        response_handler=lambda c: c["result"],
    )

    host = endpoint(
        "AuditApi.host",
        method="POST",
        url="/api/v4/audit/host/",
        params={
            "software": Annotated[
                list[AuditItem | str],
                Field(
                    min_length=1,
                    description="List of dicts. E.g., [{'product': 'curl', 'version': '8.11.1', ...}, ...]",
                ),
            ],
            "application": Annotated[AuditItem | str, Field(default=Unset)],
            "operating_system": Annotated[AuditItem | str, Field(default=Unset)],
            "hardware": Annotated[AuditItem | str, Field(default=Unset)],
            "match": Annotated[Literal["partial", "full"], Field(default="partial")],
            "fields": Annotated[
                Sequence[AuditFields],
                Field(
                    default=Unset,
                    description="List of fields to retrieve about each vulnerability",
                ),
            ],
            "config": Annotated[Sequence[str], Field(default=Unset)],
            "catalog": Annotated[Literal["official", "extended"], Field(default="official")],
        },
        response_handler=lambda c: c["result"],
    )

    os_audit = endpoint(
        "AuditApi.os_audit",
        method="POST",
        url="/api/v3/audit/audit/",
        description=(
            "Linux Audit API for analyzing package vulnerabilities.\n"
            "Accepts RPM and DEB based package lists.\n"
            "For collecting RPM use command: rpm -qa --qf '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\\\\n'\n"
            "For collecting DEB use command: dpkg-query -W -f='${Package} ${Version} ${Architecture}\\\\n'\n"
        ),
        deprecated=(
            "AuditApi.os_audit() is deprecated and will be removed in future releases. "
            "Use AuditApi.linux_audit() instead."
        ),
        params={
            "os": Annotated[
                str,
                Field(description="Full name of the OS. Like Ubuntu, Debian, rhel, oraclelinux"),
            ],
            "version": Annotated[str, Field(description="OS version")],
            "packages": Annotated[
                list[str],
                Field(
                    min_length=1,
                    alias="package",
                    description="List of the installed packages",
                ),
            ],
        },
    )

    linux_audit = endpoint(
        "AuditApi.linux_audit",
        method="POST",
        url="/api/v4/audit/linux",
        description=(
            "Linux Audit API for analyzing package vulnerabilities.\n"
            "Accepts RPM, DEB and APK based package lists.\n"
            "For collecting RPM use command: rpm -qa --qf '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\\\\n'\n"
            "For collecting DEB use command: dpkg-query -W -f='${Package} ${Version} ${Architecture}\\\\n'\n"
        ),
        params={
            "os_name": Annotated[
                str,
                Field(
                    alias="osName",
                    description="Full name of the OS or OS ID. Like ubuntu, debian, rhel, ol, alpine and etc",
                ),
            ],
            "os_version": Annotated[str, Field(alias="osVersion", description="OS version")],
            "packages": Annotated[
                list[str],
                Field(
                    min_length=1,
                    max_length=2500,
                    description="List of the installed packages",
                ),
            ],
            "os_arch": Annotated[
                str | None,
                Field(
                    default=None,
                    alias="osArch",
                    description="OS architecture, default arch for packages",
                ),
            ],
            "include_unofficial": Annotated[
                bool,
                Field(
                    default=False,
                    alias="includeUnofficial",
                    description="Include unofficial packages",
                ),
            ],
            "include_candidates": Annotated[
                bool,
                Field(
                    default=False,
                    alias="includeCandidates",
                    description="Include 'candidate' vulnerabilities",
                ),
            ],
            "include_any_version": Annotated[
                bool,
                Field(
                    default=False,
                    alias="includeAnyVersion",
                    description="Include 'any' version vulnerabilities",
                ),
            ],
            "cvelist_metrics": Annotated[
                bool,
                Field(
                    default=False,
                    alias="cvelistMetrics",
                    description="Add cvelist metrics to the response, only for non free, trial licenses",
                ),
            ],
        },
    )

    sbom_audit = endpoint(
        "AuditApi.sbom_audit",
        method="POST",
        url="/api/v4/audit/sbom",
        description=("SBOM Audit API for analyzing package vulnerabilities.\n"),
        params={
            "file": Annotated[
                FilePath,
                Field(description="Path to the SBOM file"),
            ]
        },
    )

    kb_audit = endpoint(
        "AuditApi.kb_audit",
        method="POST",
        url="/api/v3/audit/kb/",
        description="Windows KB audit function",
        params={
            "os": Annotated[
                str, Field(description="Windows OS name, like 'Windows Server 2012 R2'")
            ],
            "kb_list": Annotated[
                list[str],
                Field(
                    alias="kbList",
                    min_length=1,
                    description="List of installed KB's, ['KB2918614', 'KB2918616']",
                ),
            ],
        },
    )

    win_audit = endpoint(
        "AuditApi.win_audit",
        method="POST",
        url="/api/v3/audit/winaudit/",
        description="Windows KB and software audit function",
        params={
            "os": Annotated[
                str, Field(description="Windows OS name, like 'Windows Server 2012 R2'")
            ],
            "os_version": Annotated[
                str, Field(description="Windows OS version, like '10.0.19045'")
            ],
            "kb_list": Annotated[
                list[str],
                Field(
                    description="List of installed KB's, ['KB2918614', 'KB2918616']",
                ),
            ],
            "software": Annotated[
                list[WinAuditItem],
                Field(
                    description="List of the software dicts, {'software': 'Microsoft Edge', 'version': '107.0.1418.56'}",
                ),
            ],
            "platform": Annotated[str, Field(default=Unset, description="OS platform like x86")],
        },
        add_api_key=True,
    )



from typing import Annotated

from pydantic import Field

from ..base import Unset, VulnersApiProxy, endpoint


class StixApi(VulnersApiProxy):
    make_bundle_by_id = endpoint(
        "StixApi.bundle",
        description="Make bundle of STIX objects for the given bulletin ID",
        method="GET",
        url="/api/v4/stix/bundle",
        params={
            "id": Annotated[str, Field(description="Bulletin ID")],
            "opencti_id": Annotated[
                str | None,
                Field(default=None, description="Existing OpenCTI object ID"),
            ],
        },
        response_handler=lambda resp: resp["result"],
    )



from typing import Annotated, Literal

from pydantic import Field

from ..base import VulnersApiProxy, endpoint

BulletinField = Literal[
    "title",
    "short_description",
    "type",
    "published",
    "modified",
    "href",
    "metrics",
    "exploitation",
    "cvelist",
    "ai_score",
    "epss",
    "description",
    "enchantments",
    "webApplicability",
    "cvelistMetrics",
]

DEFAULT_BULLETIN_FIELDS: list[BulletinField] = [
    "title",
    "short_description",
    "type",
    "href",
    "published",
    "modified",
    "ai_score",
]


class SubscriptionV4Api(VulnersApiProxy):
    get_list = endpoint(
        "SubscriptionV4Api.list",
        method="GET",
        url="/api/v4/subscriptions/list/",
    )

    get = endpoint(
        "SubscriptionV4Api.get",
        method="GET",
        url="/api/v4/subscriptions/get/",
        params={"id": str},
    )

    create = endpoint(
        "SubscriptionV4Api.create",
        method="POST",
        url="/api/v4/subscriptions/create/",
        params={
            "name": str,
            "query": dict,
            "delivery": dict,
            "licenseId": Annotated[str | None, Field(default=None)],
            "bulletin_fields": Annotated[
                list[BulletinField], Field(default=DEFAULT_BULLETIN_FIELDS)
            ],
            "is_active": Annotated[bool, Field(default=True)],
            "timestamp_source": Annotated[
                Literal[
                    "modified",
                    "published",
                    "timestamps.created",
                    "timestamps.updated",
                    "timestamps.enriched",
                    "timestamps.reviewed",
                    "timestamps.metricsUpdated",
                    "timestamps.webApplicabilityUpdated",
                ],
                Field(default="modified"),
            ],
            "send_empty_result": Annotated[bool, Field(default=False)],
        },
    )

    update = endpoint(
        "SubscriptionV4Api.update",
        method="PUT",
        url="/api/v4/subscriptions/update/",
        params={
            "id": str,
            "name": str,
            "query": dict,
            "delivery": dict,
            "licenseId": Annotated[str | None, Field(default=None)],
            "bulletin_fields": Annotated[
                list[BulletinField], Field(default=DEFAULT_BULLETIN_FIELDS)
            ],
            "is_active": Annotated[bool, Field(default=True)],
            "timestamp_source": Annotated[
                Literal[
                    "modified",
                    "published",
                    "timestamps.created",
                    "timestamps.updated",
                    "timestamps.enriched",
                    "timestamps.reviewed",
                    "timestamps.metricsUpdated",
                    "timestamps.webApplicabilityUpdated",
                ],
                Field(default="modified"),
            ],
            "send_empty_result": Annotated[bool, Field(default=False)],
        },
    )

    delete = endpoint(
        "SubscriptionV4Api.delete",
        method="DELETE",
        url="/api/v4/subscriptions/delete/",
        params={
            "id": str,
        },
    )



from typing import Any, Literal

from ..base import VulnersApiProxy, endpoint


class WebhookApi(VulnersApiProxy):
    list = endpoint(
        "WebhookApi.list",
        method="GET",
        url="/api/v3/subscriptions/listWebhookSubscriptions/",
        response_handler=lambda c: c["subscriptions"],
        add_api_key=True,
    )

    add = endpoint(
        "WebhookApi.add",
        method="POST",
        url="/api/v3/subscriptions/addWebhookSubscription/",
        params={
            "query": str,
        },
        add_api_key=True,
    )

    __enable = endpoint(
        "WebhookApi.enable",
        method="POST",
        url="/api/v3/subscriptions/editWebhookSubscription/",
        params={
            "subscriptionid": str,
            "active": Literal["true", "false"],
        },
        add_api_key=True,
    )

    def enable(self, id: str, active: bool) -> dict[str, Any]:
        return self.__enable(id, "true" if active else "false")

    delete = endpoint(
        "WebhookApi.delete",
        method="POST",
        url="/api/v3/subscriptions/removeWebhookSubscription/",
        params={
            "subscriptionid": str,
        },
        add_api_key=True,
    )

    __read = endpoint(
        "WebhookApi.read",
        method="GET",
        url="/api/v3/subscriptions/webhook",
        params={
            "subscriptionid": str,
            "newest_only": Literal["true", "false"],
        },
        add_api_key=True,
    )

    def read(self, id: str, newest_only: bool = True) -> dict[str, Any]:
        return self.__read(id, "true" if newest_only else "false")



from typing import Annotated, Literal

from pydantic import Field

from ..base import Unset, VulnersApiProxy, endpoint


class SubscriptionApi(VulnersApiProxy):
    list = endpoint(
        "SubscriptionApi.list",
        method="GET",
        url="/api/v3/subscriptions/listEmailSubscriptions/",
        response_handler=lambda c: c["subscriptions"],
        add_api_key=True,
        deprecated=(
            "SubscriptionApi.list() is deprecated and will be removed in future releases.\n"
        ),
    )

    add = endpoint(
        "SubscriptionApi.add",
        method="POST",
        url="/api/v3/subscriptions/addEmailSubscription/",
        params={
            "query": str,
            "email": str,
            "format": Annotated[Literal["html", "json", "pdf"], Field(default="html")],
            "crontab": Annotated[str, Field(default=Unset)],
            "query_type": Annotated[str, Field(default="lucene")],
        },
        add_api_key=True,
        deprecated=(
            "SubscriptionApi.add() is deprecated and will be removed in future releases.\n"
        ),
    )

    edit = endpoint(
        "SubscriptionApi.edit",
        method="POST",
        url="/api/v3/subscriptions/editEmailSubscription/",
        params={
            "subscriptionid": str,
            "format": Annotated[Literal["html", "json", "pdf"], Field(default=Unset)],
            "crontab": Annotated[str, Field(default=Unset)],
            "active": Annotated[Literal["yes", "no", "true", "false"], Field(default=Unset)],
        },
        add_api_key=True,
        deprecated=(
            "SubscriptionApi.edit() is deprecated and will be removed in future releases.\n"
        ),
    )

    delete = endpoint(
        "SubscriptionApi.delete",
        method="POST",
        url="/api/v3/subscriptions/removeEmailSubscription/",
        params={
            "subscriptionid": str,
        },
        add_api_key=True,
        deprecated=(
            "SubscriptionApi.delete() is deprecated and will be removed in future releases.\n"
        ),
    )



from typing import Annotated, Any, Literal

from pydantic import Field

from ..base import VulnersApiProxy, endpoint


class ReportApi(VulnersApiProxy):
    __report = endpoint(
        "Report.__report",
        method="POST",
        url="/api/v3/reports/vulnsreport",
        params={
            "reporttype": Literal[
                "vulnssummary", "vulnslist", "ipsummary", "scanlist", "hostvulns"
            ],
            "skip": Annotated[
                int,
                Field(
                    ge=0,
                    lt=10000,
                    description="Skip this amount of items. 10000 is the hard limit",
                ),
            ],
            "size": Annotated[
                int,
                Field(
                    gt=0,
                    le=10000,
                    description="The maximum number of items to return. 10000 is the hard limit",
                ),
            ],
            "filter": Annotated[
                dict[str, Any],
                Field(
                    description="Dict of fields to filter, eg { 'OS': 'Centos', 'OSVersion': '7'}",
                ),
            ],
            "sort": Annotated[
                str, Field(description="Field to sort, eg 'severity' or '-severity' to sort desc")
            ],
        },
        response_handler=lambda c: c["report"],
    )

    def vulns_summary(
        self,
        limit: int = 30,
        offset: int = 0,
        filter: dict[str, Any] | None = None,
        sort: str = "",
    ) -> Any:
        """
        Get Linux Audit results. Return summary for all found vulnerabilities - id, title, score, severity etc

        limit: The maximum number of items to return. 10000 is the hard limit.
        offset: Skip this amount of items. 10000 is the hard limit.
        filter: Dict of fields to filter, eg { 'OS': 'Centos', 'OSVersion': '7'}
        sort: Field to sort, eg 'severity' or '-severity' to sort desc
        """

        return self.__report("vulnssummary", offset, limit, filter or {}, sort)

    def vulns_list(
        self,
        limit: int = 30,
        offset: int = 0,
        filter: dict[str, Any] | None = None,
        sort: str = "",
    ) -> Any:
        """
        Get Linux Audit results. Return list of vulnerabilities found on hosts:
            vulnerability id, vulnerability title, vulnerability severity, host information,  etc

        limit: The maximum number of items to return. 10000 is the hard limit.
        offset: Skip this amount of items. 10000 is the hard limit.
        filter: Dict of fields to filter, eg { 'OS': 'Centos', 'OSVersion': '7'}
        sort: Field to sort, eg 'severity' or '-severity' to sort desc
        """

        return self.__report("vulnslist", offset, limit, filter or {}, sort)

    def ip_summary(
        self,
        limit: int = 30,
        offset: int = 0,
        filter: dict[str, Any] | None = None,
        sort: str = "",
    ) -> Any:
        """
        Get Linux Audit results. Return summary for hosts:
            agent id, host ip, host fqdn, os name and version, found vulnerabilities count and severity

        limit: The maximum number of items to return. 10000 is the hard limit.
        offset: Skip this amount of items. 10000 is the hard limit.
        filter: Dict of fields to filter, eg { 'OS': 'Centos', 'OSVersion': '7'}
        sort: Field to sort, eg 'total' or '-total' to sort desc
        """

        return self.__report("ipsummary", offset, limit, filter or {}, sort)

    def scan_list(
        self,
        limit: int = 30,
        offset: int = 0,
        filter: dict[str, Any] | None = None,
        sort: str = "",
    ) -> Any:
        """
        Get Linux Audit results. Return list of scans:
           host ip and fqdn, os name and version, scan date, cvss score

        limit: The maximum number of items to return. 10000 is the hard limit.
        offset: Skip this amount of items. 10000 is the hard limit.
        filter: Dict of fields to filter, eg { 'OS': 'Centos', 'OSVersion': '7'}
        sort: Field to sort, eg 'modified' or '-modified' to sort desc
        """

        return self.__report("scanlist", offset, limit, filter or {}, sort)

    def host_vulns(
        self,
        limit: int = 30,
        offset: int = 0,
        filter: dict[str, Any] | None = None,
        sort: str = "",
    ) -> Any:
        """
        Get Linux Audit results. Return list of hosts and host vulnerabilities:
           host ip and fqdn, os name and version, cumulative fix, vulnerability ids

        limit: The maximum number of items to return. 10000 is the hard limit.
        offset: Skip this amount of items. 10000 is the hard limit.
        filter: Dict of fields to filter, eg { 'OS': 'Centos', 'OSVersion': '7'}
        sort: Field to sort, eg 'modified' or '-modified' to sort desc
        """

        return self.__report("hostvulns", offset, limit, filter or {}, sort)



from __future__ import annotations

import pathlib
import re
import sys
import textwrap
import warnings
import zlib
from collections import defaultdict
from functools import wraps
from importlib.metadata import version
from time import sleep, time
from typing import Any, Callable, ClassVar, Literal, Mapping

import httpx
import orjson
from pydantic import ConfigDict, create_model
from pydantic.fields import FieldInfo
from typing_inspection import introspection

__version__ = version("vulners")


class VulnersApiError(Exception):
    def __init__(self, http_status, data):
        super(VulnersApiError, self).__init__(data)
        self.http_status = http_status


class RateLimitBucket:
    """An implementation of the Token Bucket algorithm."""

    def __init__(self, rate=10.0, burst=1.0):
        self._rate = float(rate)
        self._burst = min(float(burst), self._rate)
        self._allowance = self._burst
        self._last_check = time()

    def update(self, rate, burst=1.0):
        self._rate = float(rate)
        self._burst = min(float(burst), self._rate)

    def consume(self):
        while 1:
            now = time()
            # number of seconds since the last call
            delta = now - self._last_check
            self._last_check = now
            # increase the number of allowed calls
            self._allowance += delta * self._rate
            if self._allowance > self._burst:
                # don't allow more than "burst" calls
                self._allowance = self._burst
            if self._allowance < 1:
                # cold down
                sleep((1 - self._allowance) / self._rate)
                continue
            self._allowance -= 1
            break


class VulnersApiTransport(httpx.BaseTransport):
    def __init__(self, transport: httpx.BaseTransport):
        self.transport = transport

    def handle_request(self, request: httpx.Request) -> httpx.Response:
        response = self.transport.handle_request(request)
        if "set-cookie" in response.headers:
            del response.headers["set-cookie"]
        return response


class VulnersApiProxy:
    def __init__(self, base: VulnersApiBase):
        self._invoke = base._invoke


class VulnersApiBase:
    _ratelimit_key: ClassVar[str] = ""

    _ratelimits: dict[str, RateLimitBucket] = defaultdict(RateLimitBucket)

    def __init__(
        self,
        api_key: str,
        proxy: str | None = None,
        *,
        retry_count: int = 3,
        server_url: str = "https://vulners.com",
        timeout: float = 60.0,
    ):
        """
        Create API.

        :param api_key:
            Vulners API key. You can get one from https://vulners.com
        :param proxy:
            Proxy url, example: "https://myproxy.com:3128"
        """
        if not api_key:
            raise ValueError(
                "API key must be provided. You can obtain one for free at https://vulners.com"
            )

        if not isinstance(api_key, str):
            raise TypeError("api_key parameter must be a string value")

        self._client = httpx.Client(
            follow_redirects=True,
            base_url=server_url,
            transport=VulnersApiTransport(httpx.HTTPTransport(proxy=proxy, retries=retry_count)),
            headers={
                "User-Agent": "Vulners Python API %s" % __version__,
                "X-Api-Key": api_key,
            },
            timeout=httpx.Timeout(timeout),
        )
        self._api_key = api_key

    def _invoke(
        self,
        method: Literal["GET", "POST", "PUT", "DELETE", "PATCH"],
        url: str,
        params: dict[str, Any],
        path_params: tuple[str, ...],
        file_params: list[str] | None = None,
        add_api_key: bool = False,
        timeout: float | None = None,
        parse_content: bool = True,
    ):
        if path_params:
            url = re.sub("{([^}]*)}", lambda m: str(params.pop(m.group(1))), url)
        if add_api_key:
            params["apiKey"] = self._api_key
        files = {}
        if file_params:
            for file_param in file_params:
                file_path = params.pop(file_param, None)
                if file_path:
                    files[file_param] = open(file_path, "rb")
        if files:
            kwargs: dict[str, Any] = {"data": params, "files": files}
        else:
            kwargs = {"params" if method in ("GET", "DELETE") else "json": params}
        if timeout is not None:
            kwargs["timeout"] = timeout
        bucket = self._ratelimits[self._ratelimit_key or url]
        bucket.consume()
        response = self._client.request(method, url, **kwargs)
        if files:
            for file in files.values():
                file.close()
        limit = response.headers.get("X-Vulners-Ratelimit-Reqlimit")
        if limit:
            try:
                bucket.update(rate=float(limit) / 60.0)
            except (TypeError, ValueError):
                pass
        content = response.content
        if response.headers["content-type"] == "application/json":
            result = orjson.loads(content)
            if isinstance(result, dict) and "data" in result:
                data = result["data"]
                if isinstance(data, dict) and data.get("error"):
                    raise VulnersApiError(response.status_code, data)
                return data
            if response.status_code >= 400:
                raise VulnersApiError(response.status_code, result)
            return result

        if response.headers["content-type"] == "application/x-gzip-compressed":
            content = zlib.decompress(content, wbits=31)
        elif response.headers["content-type"] == "application/x-zip-compressed":
            import io
            import zipfile

            with zipfile.ZipFile(io.BytesIO(content)) as z:
                if len(z.namelist()) > 1:
                    raise RuntimeError("Unexpected file count in Vulners ZIP archive")
                filename = z.namelist()[0]
                content = z.open(filename).read()
        if parse_content:
            return orjson.loads(content)
        return content


def _ann_repr(t: Any) -> str:
    if isinstance(t, type):
        if t.__module__ != "builtins":
            return f"{t.__module__}.{t.__name__}"
        return t.__name__
    return repr(t)


def deprecation_warning(text: str) -> None:
    text = textwrap.indent(text, "[!] ")
    warnings.warn(
        f"\n[!] DEPRECATION WARNING\n{text}",
        DeprecationWarning,
    )


def deprecated(message: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
    def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
        @wraps(func)
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            deprecation_warning(message)
            return func(*args, **kwargs)

        return wrapper

    return decorator


def endpoint(
    name: str,
    /,
    method: Literal["GET", "POST", "PUT", "DELETE", "PATCH"],
    url: str,
    description: str | None = None,
    params: Mapping[str, Any] | None = None,
    response_handler: Callable[[Any], Any] | None = None,
    parse_response: bool = True,
    wrapper: Any = None,
    add_api_key: bool = False,
    deprecated: str | None = None,
    timeout: float | None = None,
) -> Callable[..., Any]:
    assert method in ("GET", "POST", "PUT", "DELETE", "PATCH")
    assert isinstance(url, str)
    assert description is None or isinstance(description, str)
    assert params is None or isinstance(params, Mapping)

    parent_namespace = sys._getframe(1).f_globals
    params = params or {}

    path_params = tuple(re.findall("{([^}]*)}", url))
    for param in path_params:
        assert param in params

    if wrapper is not None:
        returns = f"{wrapper.__module__}.{wrapper.__name__}"
    else:
        returns = "dict[str, typing.Any]"

    func_args = []
    file_params = []
    for param, param_type in params.items():
        ann = introspection.inspect_annotation(
            param_type, annotation_source=introspection.AnnotationSource.FUNCTION
        )
        if ann.metadata:
            metadata = [x for x in ann.metadata if isinstance(x, FieldInfo)][0]
        else:
            metadata = None
        if ann.type is pathlib.Path:
            file_params.append((metadata.alias or param) if metadata else param)
        if metadata and not metadata.is_required():
            func_args.append(f"{param}: {_ann_repr(ann.type)} = {ann.metadata[0].default!r}")
        else:
            func_args.append(f"{param}: {_ann_repr(ann.type)}")

    call_args = "{" + ", ".join(f"{name!r}: {name}" for name in params) + "}"
    model = create_model(
        name, **params, __config__=ConfigDict(extra="forbid", validate_by_name=True)
    )
    namespace: dict[str, Any] = {"__model": model, "Unset": Unset}
    code = "\n".join(
        [
            f"def endpoint({', '.join(func_args)}) -> {returns}:",
            f"    _callargs = {{ _k: _v for _k, _v in {call_args}.items() if _v is not Unset }}",
            f"    return __model(**_callargs).model_dump(mode='json', exclude_unset=True, by_alias=True)",
        ]
    )
    exec(code, namespace)
    endpoint = namespace["endpoint"]
    endpoint.__name__ = name.rsplit(".", 1)[1] if "." in name else name
    endpoint.__qualname__ = name
    endpoint.__module__ = parent_namespace["__name__"]

    @wraps(endpoint)
    def func(api: VulnersApiBase, *args: Any, **kwargs: Any) -> Any:
        if deprecated:
            deprecation_warning(deprecated)

        params = endpoint(*args, **kwargs)
        content = api._invoke(
            method, url, params, path_params, file_params, add_api_key, timeout, parse_response
        )
        if response_handler:
            content = response_handler(content)
        if wrapper is not None:
            content = wrapper(api, content)
        return content

    func.__doc__ = description
    return func


class ResultSet(list):
    total = None

    @classmethod
    def from_dataset(cls, data, total):
        ret = cls(data)
        ret.total = total
        return ret


class _Unset:
    def __repr__(self) -> str:
        return "Unset"


Unset = _Unset()



import re
from typing import Annotated, Any, Literal, Sequence

from pydantic import Field

from ..base import ResultSet, Unset, VulnersApiProxy, endpoint

DEFAULT_FIELDS = (
    "id",
    "title",
    "description",
    "type",
    "bulletinFamily",
    "cvss",
    "published",
    "modified",
    "lastseen",
    "href",
    "sourceHref",
    "sourceData",
    "cvelist",
    "vulnStatus",
    "assigned",
)


class SearchApi(VulnersApiProxy):
    __search = endpoint(
        "SearchApi.search",
        method="POST",
        url="/api/v3/search/lucene/",
        params={
            "query": str,
            "size": Annotated[int, Field(gt=0, le=10000)],
            "skip": Annotated[int, Field(ge=0, le=10000)],
            "fields": Sequence[str],
        },
    )

    def search_bulletins(
        self,
        query: str,
        limit: int = 20,
        offset: int = 0,
        fields: Sequence[str] = DEFAULT_FIELDS,
    ) -> ResultSet:
        """
        Search in Vulners database.

        query: Vulners query. See https://vulners.com/help for the details.
        limit: The maximum number of documents to return.
        offset: The number of documents to skip in the result set.
        fields: List of fields to return.

        Returns list of the documents.
        Use .total to get the total number of found documents.
        """
        limit = min(limit, 10000 - offset)  # real limit is unknown
        search = self.__search(query, limit, offset, fields)
        return ResultSet.from_dataset([e["_source"] for e in search["search"]], search["total"])

    def search_bulletins_all(
        self,
        query: str,
        limit: int = 100,  # default limit is 100
        fields: Sequence[str] = DEFAULT_FIELDS,
    ) -> ResultSet:
        offset = 0
        limit = min(limit, 10000)
        chunk = self.__search(query, limit, offset, fields)
        total = min(limit, chunk["total"])
        max_chunk_size = chunk["maxSearchSize"]
        result = ResultSet.from_dataset([e["_source"] for e in chunk["search"]], chunk["total"])
        offset += len(result)
        while offset < total:
            chunk_size = min(max_chunk_size, total - offset)
            chunk = self.__search(query, chunk_size, offset, fields)
            data = chunk["search"]
            result += [e["_source"] for e in data]
            if len(data) < chunk_size:
                break
            offset += len(data)
        return result

    @staticmethod
    def __get_exploit_query(query: str, lookup_fields: Sequence[str] | None = None) -> str:
        if re.match(r"^CVE-\d{4}-\d+$", (query := query.strip()), re.IGNORECASE):
            query = f'"{query}"'
        if lookup_fields:
            return "bulletinFamily:exploit AND (%s)" % (
                " OR ".join('%s:"%s"' % (field, query) for field in lookup_fields)
            )
        else:
            return "bulletinFamily:exploit AND (%s)" % query

    def search_exploits(
        self,
        query: str,
        lookup_fields: Sequence[str] | None = None,
        limit: int = 20,
        offset: int = 0,
        fields: Sequence[str] = DEFAULT_FIELDS,
    ) -> ResultSet:
        """
        Search in Vulners database for the exploits.

        query: Software name and criteria.
        lookup_fields: Make a strict search using lookup fields. Like ["title"]
        limit: The maximum number of documents to return.
        offset: The number of documents to skip in the result set.
        fields: List of fields to return.

        Returns list of the documents.
        Use .total to get the total number of found documents.
        """
        return self.search_bulletins(
            self.__get_exploit_query(query, lookup_fields),
            limit=limit,
            offset=offset,
            fields=fields,
        )

    def search_exploits_all(
        self,
        query: str,
        lookup_fields: Sequence[str] | None = None,
        limit: int = 100,  # default limit
        fields: Sequence[str] = DEFAULT_FIELDS,
    ) -> ResultSet:
        return self.search_bulletins_all(
            self.__get_exploit_query(query, lookup_fields), limit=limit, fields=fields
        )

    __get_bulletins = endpoint(
        "SearchApi.__get_bulletins",
        method="POST",
        url="/api/v3/search/id/",
        description="Fetch multiple bulletins by ids.",
        params={
            "id": Annotated[
                Sequence[str],
                Field(min_length=1, description="List of ID's. E.g., ['CVE-2017-14174']"),
            ],
            "fields": Annotated[Sequence[str], Field(default=DEFAULT_FIELDS)],
            "references": Annotated[bool, Field(default=False)],
        },
    )

    def get_multiple_bulletins(
        self, id: Sequence[str], fields: Sequence[str] = DEFAULT_FIELDS, references: bool = True
    ) -> dict[str, Any]:
        return self.__get_bulletins(id, fields=fields, references=references)["documents"]

    def get_bulletin(self, id: str, fields: Sequence[str] = DEFAULT_FIELDS) -> dict[str, Any]:
        return self.get_multiple_bulletins([id], fields).get(id, {})

    def get_multiple_bulletin_references(self, id: Sequence[str]) -> dict[str, Any]:
        return self.__get_bulletins(id, fields=[], references=True)["references"]

    def get_bulletin_references(self, id: str) -> dict[str, Any]:
        return self.get_multiple_bulletin_references([id]).get(id, {})

    def get_multiple_bulletins_with_references(
        self, id: Sequence[str], fields: Sequence[str] = DEFAULT_FIELDS
    ) -> dict[str, Any]:
        return self.__get_bulletins(id, fields=fields, references=True)

    def get_bulletin_with_references(
        self, id: str, fields: Sequence[str] = DEFAULT_FIELDS
    ) -> dict[str, Any]:
        return self.get_multiple_bulletins_with_references([id], fields=fields)

    def get_kb_seeds(self, kbid: str) -> dict[str, Any]:
        """
        Returns superseeds and parentseeds for the given KB.
        Superseeds means "KB which are covered by this KB".
        Parentseeds means "KB which are covering this KB".

        superseeds --> KB --> parentseeds

        kbid: Microsoft KB identificator
        """
        candidate = self.get_bulletin(id=kbid, fields=["superseeds", "parentseeds"])
        if candidate:
            return {
                "superseeds": candidate.get("superseeds", []),
                "parentseeds": candidate.get("parentseeds", []),
            }
        return {}

    def get_kb_updates(self, kbid: str, fields: Sequence[str] = DEFAULT_FIELDS):
        """
        Returns list of updates for KB.

        kbid: Microsoft KB identificator.
        """
        query = "type:msupdate AND kb:(%s)" % kbid
        return self.search_bulletins(query, limit=1000, fields=fields)

    get_bulletin_history = endpoint(
        "SearchApi.get_bulletin_history",
        method="GET",
        url="/api/v3/search/history/",
        description="Bulletin history list",
        params={
            "id": str,
        },
        response_handler=lambda c: c["result"],
    )

    get_web_vulns = endpoint(
        "SearchApi.get_web_vulns",
        method="POST",
        url="/api/v4/search/web-vulns/",
        description="Search vulnerabilities related to web vulnerabilities",
        params={
            "paths": list[str],
            "application": Annotated[str | dict[str, Any] | None, Field(default=None)],
            "match": Annotated[Literal["partial", "full"], Field(default="partial")],
            "config": Annotated[Sequence[str], Field(default=Unset)],
            "catalog": Annotated[Literal["official", "extended"], Field(default="official")],
        },
    )



from functools import cached_property
from typing import Any

from ..base import VulnersApiBase, deprecated
from .archive import ArchiveApi
from .audit import AuditApi
from .misc import MiscApi
from .report import ReportApi
from .search import SearchApi
from .stix import StixApi
from .subscription import SubscriptionApi
from .subscription_v4 import SubscriptionV4Api
from .webhook import WebhookApi


class VulnersApi(VulnersApiBase):
    @cached_property
    def search(self) -> SearchApi:
        return SearchApi(self)

    @cached_property
    def stix(self) -> StixApi:
        return StixApi(self)

    @deprecated(
        "VulnersApi.find() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.search_bulletins() instead."
    )
    def find(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.search_bulletins(*args, **kwargs)

    @deprecated(
        "VulnersApi.find_all() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.search_bulletins_all() instead."
    )
    def find_all(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.search_bulletins_all(*args, **kwargs)

    @deprecated(
        "VulnersApi.find_exploit() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.search_exploits() instead."
    )
    def find_exploit(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.search_exploits(*args, **kwargs)

    @deprecated(
        "VulnersApi.find_exploit_all() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.search_exploits_all() instead."
    )
    def find_exploit_all(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.search_exploits_all(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_multiple_bulletins() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_multiple_bulletins() instead."
    )
    def get_multiple_bulletins(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_multiple_bulletins(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_bulletin() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_bulletin() instead."
    )
    def get_bulletin(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_bulletin(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_multiple_bulletin_references() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_multiple_bulletin_references() instead."
    )
    def get_multiple_bulletin_references(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_multiple_bulletin_references(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_bulletin_references() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_bulletin_references() instead."
    )
    def get_bulletin_references(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_bulletin_references(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_multiple_documents_with_references() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_multiple_bulletins_with_references() instead."
    )
    def get_multiple_documents_with_references(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_multiple_bulletins_with_references(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_document_with_references() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_bulletin_with_references() instead."
    )
    def get_document_with_references(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_bulletin_with_references(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_kb_seeds() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_kb_seeds() instead."
    )
    def get_kb_seeds(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_kb_seeds(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_kb_updates() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_kb_updates() instead."
    )
    def get_kb_updates(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_kb_updates(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_bulletin_history() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.search.get_bulletin_history() instead."
    )
    def get_bulletin_history(self, *args: Any, **kwargs: Any) -> Any:
        return self.search.get_bulletin_history(*args, **kwargs)

    @cached_property
    def misc(self) -> MiscApi:
        return MiscApi(self)

    @deprecated(
        "VulnersApi.get_web_application_rules() is deprecated and will be removed in future releases.\n"
    )
    def get_web_application_rules(self, *args: Any, **kwargs: Any) -> Any:
        return self.misc.get_web_application_rules(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_suggestion() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.misc.get_suggestion() instead."
    )
    def get_suggestion(self, *args: Any, **kwargs: Any) -> Any:
        return self.misc.get_suggestion(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_ai_score() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.misc.get_ai_score() instead."
    )
    def get_ai_score(self, *args: Any, **kwargs: Any) -> Any:
        return self.misc.get_ai_score(*args, **kwargs)

    @deprecated(
        "VulnersApi.query_autocomplete() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.misc.query_autocomplete() instead."
    )
    def query_autocomplete(self, *args: Any, **kwargs: Any) -> Any:
        return self.misc.query_autocomplete(*args, **kwargs)

    @deprecated(
        "VulnersApi.search_cpe() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.misc.search_cpe() instead."
    )
    def search_cpe(self, *args: Any, **kwargs: Any) -> Any:
        return self.misc.search_cpe(*args, **kwargs)

    @cached_property
    def audit(self) -> AuditApi:
        return AuditApi(self)

    @deprecated(
        "VulnersApi.audit_software() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.audit.software() instead."
    )
    def audit_software(self, *args: Any, **kwargs: Any) -> Any:
        return self.audit.software(*args, **kwargs)

    @deprecated(
        "VulnersApi.audit_host() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.audit.host() instead."
    )
    def audit_host(self, *args: Any, **kwargs: Any) -> Any:
        return self.audit.host(*args, **kwargs)

    @deprecated(
        "VulnersApi.os_audit() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.audit.os_audit() instead."
    )
    def os_audit(self, *args: Any, **kwargs: Any) -> Any:
        return self.audit.os_audit(*args, **kwargs)

    @deprecated(
        "VulnersApi.kb_audit() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.audit.kb_audit() instead."
    )
    def kb_audit(self, *args: Any, **kwargs: Any) -> Any:
        return self.audit.kb_audit(*args, **kwargs)

    @deprecated(
        "VulnersApi.winaudit() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.audit.win_audit() instead."
    )
    def winaudit(self, *args: Any, **kwargs: Any) -> Any:
        return self.audit.win_audit(*args, **kwargs)

    @cached_property
    def webhook(self) -> WebhookApi:
        return WebhookApi(self)

    @deprecated(
        "VulnersApi.get_webhooks() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.webhook.list() instead."
    )
    def get_webhooks(self, *args: Any, **kwargs: Any) -> Any:
        return self.webhook.list(*args, **kwargs)

    @deprecated(
        "VulnersApi.add_webhook() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.webhook.add() instead."
    )
    def add_webhook(self, *args: Any, **kwargs: Any) -> Any:
        return self.webhook.add(*args, **kwargs)

    @deprecated(
        "VulnersApi.enable_webhook() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.webhook.enable() instead."
    )
    def enable_webhook(self, *args: Any, **kwargs: Any) -> Any:
        return self.webhook.enable(*args, **kwargs)

    @deprecated(
        "VulnersApi.delete_webhook() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.webhook.delete() instead."
    )
    def delete_webhook(self, *args: Any, **kwargs: Any) -> Any:
        return self.webhook.delete(*args, **kwargs)

    @deprecated(
        "VulnersApi.read_webhook() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.webhook.read() instead."
    )
    def read_webhook(self, *args: Any, **kwargs: Any) -> Any:
        return self.webhook.read(*args, **kwargs)

    @cached_property
    def subscription(self) -> SubscriptionApi:
        return SubscriptionApi(self)

    @cached_property
    def subscription_v4(self) -> SubscriptionV4Api:
        return SubscriptionV4Api(self)

    @deprecated(
        "VulnersApi.get_subscriptions() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.subscription_v4.get_list() instead."
    )
    def get_subscriptions(self, *args: Any, **kwargs: Any) -> Any:
        return self.subscription.list(*args, **kwargs)

    @deprecated(
        "VulnersApi.add_subscription() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.subscription_v4.create() instead."
    )
    def add_subscription(self, *args: Any, **kwargs: Any) -> Any:
        return self.subscription.add(*args, **kwargs)

    @deprecated(
        "VulnersApi.edit_subscription() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.subscription_v4.update() instead."
    )
    def edit_subscription(self, *args: Any, **kwargs: Any) -> Any:
        return self.subscription.edit(*args, **kwargs)

    @deprecated(
        "VulnersApi.delete_subscription() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.subscription_v4.delete() instead."
    )
    def delete_subscription(self, *args: Any, **kwargs: Any) -> Any:
        return self.subscription.delete(*args, **kwargs)

    @cached_property
    def report(self) -> ReportApi:
        return ReportApi(self)

    @deprecated(
        "VulnersApi.vulnssummary_report() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.report.vulns_summary() instead."
    )
    def vulnssummary_report(self, *args: Any, **kwargs: Any) -> Any:
        return self.report.vulns_summary(*args, **kwargs)

    @deprecated(
        "VulnersApi.vulnslist_report() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.report.vulns_list() instead."
    )
    def vulnslist_report(self, *args: Any, **kwargs: Any) -> Any:
        return self.report.vulns_list(*args, **kwargs)

    @deprecated(
        "VulnersApi.ipsummary_report() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.report.ip_summary() instead."
    )
    def ipsummary_report(self, *args: Any, **kwargs: Any) -> Any:
        return self.report.ip_summary(*args, **kwargs)

    @deprecated(
        "VulnersApi.scanlist_report() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.report.scan_list() instead."
    )
    def scanlist_report(self, *args: Any, **kwargs: Any) -> Any:
        return self.report.scan_list(*args, **kwargs)

    @deprecated(
        "VulnersApi.hostvulns_report() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.report.host_vulns() instead."
    )
    def hostvulns_report(self, *args: Any, **kwargs: Any) -> Any:
        return self.report.host_vulns(*args, **kwargs)

    @cached_property
    def archive(self) -> ArchiveApi:
        return ArchiveApi(self)

    @deprecated(
        "VulnersApi.get_collection() is deprecated and will be removed in future releases.\n"
        "Use VulnersApi.archive.fetch_collection() and VulnersApi.archive.fetch_collection_update() instead"
    )
    def get_collection(self, *args: Any, **kwargs: Any) -> Any:
        return self.archive.get_collection(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_distributive() is deprecated and will be removed in future releases.\n"
    )
    def get_distributive(self, *args: Any, **kwargs: Any) -> Any:
        return self.archive.get_distributive(*args, **kwargs)

    @deprecated(
        "VulnersApi.get_distributive() is deprecated and will be removed in future releases.\n"
    )
    def getsploit(self, *args: Any, **kwargs: Any) -> Any:
        return self.archive.getsploit(*args, **kwargs)
